#!/bin/bash
#
# check_hp_ilo - Display information about HP iLOs card.
#                The ILO is the Integrated Lights-Out management processor
#                used on HP ProLiant and BladeSystem servers.
# 
# Version: 1.0
#
# This script is based upon findilos and adapted for use with nagios
# Changes by Karsten Mueller <kmu@me.com>
#
# Based upon findilos
#
# Requirements: curl
#
# Author: Benedito Ramos
# Date: 10/16/2014
# email: diramos@uol.com.br
#
###############################################################
# --- FUNCTIONS ---------------------------------
###############################################################
export SCRIPT_NAME=`basename $0`
export RETRY=7
export NP=0
export NoA=$#
function usage()
{
echo
echo "Usage: $SCRIPT_NAME [Options]"
echo 
echo "Options: -H <HOST ip-address>"
echo "         -i <ILO ip-adress>"
echo "         -h for help"
echo
}
###############################################################
# --- MAIN ---------------------------------
###############################################################
if [ $NoA -lt 4 ] || [ $NoA -gt 4 ]
   then
   if [ $NoA -eq 1 ] && [ $1 = "-h" ] ; then
      usage
      exit 0
   else
      echo
      echo "Invalid number of arguments supplied"
      usage
      exit 3
   fi
fi
if [ $NoA -eq 4 ] ; then
   while getopts H:i: OPTION ; do
        case "$OPTION"
        in
            H) HOST=$OPTARG ;;
			i) IP_ILO=$OPTARG ;;
	       \?) echo
               echo "Invalid command arguments supplied"
               usage
               exit 3;;
        esac
    done
fi
HOST=$2
IP_ILO=$4
LEN_IP_ILO=$(expr length $IP_ILO - 1)
if [ $LEN_IP_ILO -eq 0 ] ; then
	echo "CRITICAL - no ILO configured on HOST"
	exit 2
fi
CHKHOST=$(ping -c 1 $HOST 2>/dev/null)
RETCO=$?
if [ $RETCO -ne 0 ] ; then
	if [ $RETCO -eq 1 ]
	   then
	   printf "UNKNOWN: Host unavailable. Wait for the next check\n"
	   exit 3
	fi
	if [ $RETCO -eq 2 ]
	   then
	   printf "UNKNOWN: Unknown host. Check the registered address\n"
	   exit 3
	fi
else
	RES_CMD=$(curl --proxy "" --fail --max-time 10 http://$IP_ILO/xmldata?item=all 2>/dev/null)
	RET_CMD=$?
	if [ $RET_CMD -gt 0 ]; then
		echo "CRITICAL: No HP ILO card detected on IP $IP_ILO"
		exit 2
	else
		FIRM_ILO=$(echo $RES_CMD | sed -n 's/^.*<FWRI>\(.*\)<\/FWRI>.*$/\1/p' | sed 's/ //g')
		VERSION_ILO=$(echo $RES_CMD | sed -n 's/^.*<PN>\(.*\)<\/PN>.*$/\1/p' | sed 's/ //g')
		SERIAL_ILO=$(echo $RES_CMD | sed -n 's/^.*<SBSN>\(.*\)<\/SBSN>.*$/\1/p' | sed 's/ //g')
		SERVER_ILO=$(echo $RES_CMD | sed -n 's/^.*<SPN>\(.*\)<\/SPN>.*$/\1/p')
		if [ "$VERSION_ILO" == "" -o "$SERIAL_ILO" == "" ] ; then
		   echo "CRITICAL: ILO card detected on IP $IP_ILO, but not an HP ILO card"
		   exit 2
		else
		   echo "OK: $VERSION_ILO - IP: $IP_ILO - Firmware: $FIRM_ILO - Server: $SERVER_ILO - Serial: $SERIAL_ILO."
		   exit 0
		fi
	fi
fi